/*
 *  Copyright (C) 2008, Pedro Carlos da Silva Lara
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.   
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
#ifndef _CURVE_H_
#define _CURVE_H_
#include <gmpxx.h>

namespace ec {

/**
 * Esta classe encapsula os parametros de uma curva eliptica
 * y^2 = x^3 + ax + b  sobre Zp
 */

class Curve {
public:
    
    /**
     * Contrutor default
     */
    
    Curve() {}
    
    /**
     * Principal Contrutor
     * @param a
     *     parametro 'a' da curva eliptica
     * @param b
     *     parametro 'b' da curva eliptica
     * @param p
     *     numero primo que sera utilizado   
     */
    
    Curve(const mpz_class& a, const mpz_class& b, const mpz_class& p ) : 
                _a( a ), _b( b ), _p( p ) {
    }
                
    /**
     * Construtor por copia 
     */
                
    Curve(const Curve& E) {
        _a = E.getA();
        _b = E.getB();
        _p = E.getP();            
    }
    
    ~Curve(){}
    
    /**
     * @return
     *     o parametro 'a' da curva eliptica
     */
    
    mpz_class getA() const { return _a; }
    /**
     * @return
     *     o parametro 'b' da curva eliptica
     */
    mpz_class getB() const { return _b; }
    
    /**
     * @return
     *     o numero primo usado pela curva
     */
    
    mpz_class getP() const { return _p; }       
    
    
    bool operator ==(const Curve& E) {
         return ( E.getA() == _a &&
                  E.getB() == _b &&
                  E.getP() == _p );
    }
    
private:
    mpz_class _a;
    mpz_class _b;
    mpz_class _p;      
};

} // namespace ec

#endif 
