/*
 *  Copyright (C) 2008, Pedro Carlos da Silva Lara
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.   
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef TRACE
#include <iostream>
using std::cout;
using std::endl;
#endif

#include "ecutil.h"
#include "point.h"

using ec::crypt::EcUtil;
using ec::Point;

vector< byte > EcUtil::toVectorBytes( const Point& P, 
                                      bool insertCurve ) {
    byte insert = 0x00;
    byte infty  = 0x00;
         
    if( P.getInfty() )
        infty = 0x01;
              
    if( insertCurve )
        insert = 0x10;
    
    vector< byte > ret;
    
    ret.push_back( insert | infty );
          
          
    if( !P.getInfty() ) {
        // Insert Point           
        string strPoint = P.getString();
        char * bytes = new char[strPoint.size()];
        bytes = (char*) strPoint.data();
          
        for(unsigned int i = 0; i < strPoint.size(); i++)
            ret.push_back( bytes[i] );
        delete [] bytes;    
        
    }      
    if( insertCurve ) {
        // Insert curve in vector
        char curveBuff[1000];
        gmp_sprintf(curveBuff, "(%Zd, %Zd, %Zd)", P.getCurve().getA().get_mpz_t(), 
                                                  P.getCurve().getB().get_mpz_t(),
                                                  P.getCurve().getP().get_mpz_t());
        for(int i = 0; curveBuff[i] != '\0'; i++)
            ret.push_back( curveBuff[i] );                                                 
            
    }             
    
    
    return ret;
}

Point EcUtil::toPoint(const vector< byte >& bytes) {
   
    byte id = bytes[0];
    bool infty = id % 2;
    id = id >> 4;
    bool insertCurve = id % 2;
	
	string strVet;
	
	for(unsigned int i = 1; i < bytes.size(); ++i )
		strVet += bytes[i];
#ifdef TRACE
	cout<<"buff = "<<strVet<<endl;
#endif
	size_t posY = 0;
	size_t posX = 0;
	string x;
	string y;
	
	
	if( !infty ) {
		posX = strVet.find(',', 1 );
		x = strVet.substr(1, posX - 1);
		
#ifdef TRACE
		cout<<"x = "<<x<<endl;
#endif
		posY = strVet.find(')', posX );
		y = strVet.substr(posX + 2, posY - (posX + 2) );	
			    
#ifdef TRACE
		cout<<"y = "<<y<<endl;
#endif
	}
	
	Curve* curve;
	string a;
	string b;
	string p;
	if(insertCurve) {
		size_t posAinit  = strVet.find('(', posX );
		size_t posAfinal = strVet.find(',', posAinit); 
		a = strVet.substr(posAinit + 1, posAfinal - ( posAinit + 1) );
#ifdef TRACE
		cout<<"a = "<<a<<endl;
#endif	
		size_t posB = strVet.find(',', posAfinal + 1);
		b = strVet.substr(posAfinal + 2, posB - (posAfinal + 2));
#ifdef TRACE
		cout<<"b = "<<b<<endl;
#endif
		size_t posP = strVet.find(')', posB );
		p = strVet.substr(posB + 2, posP - (posB + 2));
#ifdef TRACE
		cout<<"p = "<<p<<endl;
#endif
	}   
	
	curve = new Curve(mpz_class( a ), mpz_class( b ), mpz_class( p ));
	    
    if( infty )
    	return Point( *curve ); 
    else 
		return Point( mpz_class( x ), mpz_class( y ), *curve );    
    
    
    return Point();
}

vector< byte > EcUtil::toVectorBytes(const string& msg) {
	vector< byte > ret;
	char * bytes = new char[msg.size()];
    bytes = (char*) msg.data();
          
    for(unsigned int i = 0; i < msg.size(); i++)
    	ret.push_back( bytes[i] );
    delete [] bytes;
    
    return ret;
}
