/*
 *  Copyright (C) 2008, Pedro Carlos da Silva Lara
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.   
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
#ifndef _HEADER_ECUTIL_H_
#define _HEADER_ECUTIL_H_
#include <vector>
#include <string>

#include "defines.h"
#include "point.h"

using std::vector;
using std::string;

using ec::Point;
using ec::Curve;

namespace ec {
namespace crypt {
          
/**
 * Esta classe possui membros estaticos para auxiliar 
 * as classes principais. E composta basicamente 
 * de metodos que convertem dados.
 * 
 */

class EcUtil {
public:

      /**
       * Converte um objeto do tipo Point para um vector 
       * de bytes
       * 
       * @param P
       *     ponto que sera convertido
       *
       * @param insertCurve
       *     caso true insere a curva que o ponto P encasula no retorno, 
       *     caso false nao insere, no retorno a curva.
       *
       * @return
       *     o ponto convertido
       * 
       */
       
      static vector< byte > toVectorBytes( const Point& P, 
                                           bool insertCurve );
      /**
       * Converte uma mensagem para um vector de byte
       */
       
      static vector< byte > toVectorBytes(const string& msg);
      /**
       * Converte um vector de bytes para 
       * um objeto do tipo Point 
       * 
       */
       
      static Point toPoint(const vector< byte >& bytes);
      
      
            
};

} // namespace ec
} // namespace crypt

#endif // _HEADER_ECUTIL_H_
