/*
 *  Copyright (C) 2008, Pedro Carlos da Silva Lara
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.   
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
#ifndef _HEADER_KEY_H_
#define _HEADER_KEY_H_

#include <string>

using std::string;

namespace ec {
namespace crypt {
          
/**
 * Esta classe sera usada como super classe
 * para as classes que sao chaves publicas.
 * O template 'T' define o tipo de chave.
 * Ex: A chave publica do protocolo Diffie-Hellman
 * e um objeto do tipo Point
 */
 
template<typename T>
class PublicKey {
public:
 
    /**
     * Contrutor default
     */
    
    PublicKey() {}
    
    /**
     * Principal construtor
     * @param key
     *     template que constituira a chave 
     */
       
    PublicKey(const T& key) : _key(key) {}
    
    /**
     * @return
     *     a chave publica
     */
    
    T getKey() const { return _key; }
    
    /**
     * @param key
     *     novo valor para a chave
     */
    
    void setKey(const T& key){ _key = key; }

protected:
   T _key;               
};

/**
 * Esta classe sera usada como super classe
 * para as classes que sao chaves privadas.
 * O template 'T' define o tipo de chave.
 * Ex: A chave privada do protocolo Diffie-Hellman
 * e um objeto do tipo mpz_class
 */

template<typename T>
class PrivateKey {
public:
    
    /**
     * Contrutor default
     */
    
    PrivateKey() {}
    
    /**
     * Principal construtor
     * @param key
     *     template que constituira a chave 
     */
    
    PrivateKey(const T& key) : _key(key) {}
    
    /**
     * @return
     *     a chave publica
     */
    
    T getKey() const { return _key; }
    
    /**
     * @param key
     *     novo valor para a chave
     */
    
    void setKey(const T& key){ _key = key; }

protected:
          
   T _key;               
};

} // namespace ec
} // namespace crypt


#endif // _HEADER_DHKEY_H_
